﻿using System;
using System.Collections.Generic;
using System.Xml;
using System.Data;

public partial class WebApi : System.Web.UI.Page
{
    //假裝一個資料庫來源
    private static DataTable _dataStore = null;
    //將XML轉成DataTable中作為初始資料
    private DataTable getDataStore()
    {
        if (_dataStore == null || Request["init"] != null)
        {
            //建立有五個欄位的DataTable
            string[] cols = "Date,Title,Category,Hyperlink,Abstract".Split(',');
            _dataStore = new DataTable("Posts");
            _dataStore.Columns.Add("Id", typeof(int));
            foreach (string col in cols)
                _dataStore.Columns.Add(col, typeof(string));
            //以Id為Primary Key
            _dataStore.PrimaryKey = new DataColumn[] { _dataStore.Columns["Id"] };
            //由XML載入Demo資料
            XmlDocument xd = new XmlDocument();
            xd.Load(Server.MapPath("App_Data/Posts.xml"));
            int i = 0;
            foreach (XmlNode p in xd.SelectNodes("//Post"))
            {
                DataRow r = _dataStore.NewRow();
                r["Id"] = i++;
                foreach (string col in cols)
                {
                    if (col == "Abstract")
                        r[col] = p.InnerText;
                    else
                        r[col] = p.Attributes[col].Value;
                }
                _dataStore.Rows.Add(r);
            }
        }
        return _dataStore;
    }

    //資料物件，用來作JSON轉換用
    public class PostItem
    {
        public string Id; //識別碼
        public string Date; //日期
        public string Category; //分類
        public string Title; //標題
        public string Hyperlink; //超連結
        public string Abstract; //摘要
        //直接將DataRow轉成資料物件，可選擇含不含摘要
        //故可同時作為清單及編輯兩用
        public PostItem(DataRow data, bool includeAbstract)
        {
            Id = data["Id"].ToString();
            Date = data["Date"].ToString();
            Category = data["Category"].ToString();
            Title = data["Title"].ToString();
            Hyperlink = data["Hyperlink"].ToString();
            if (includeAbstract) 
                Abstract = data["Abstract"].ToString();
        }
        //配合新增資料的邏輯
        public PostItem(string id)
        {
            Id = id;
            Date = DateTime.Today.ToString("yyyy/MM/dd");
            Category = "jQuery";
            Title = "新文章";
            Hyperlink = "http://blog.darkthread.net";
            Abstract = "請輸入摘要...";
        }
    }

    protected void Page_Load(object sender, EventArgs e)
    {
        //傳回結果可能包含使用者輸入結果，限定POST
        //減少被當作<script src="..." />進行XSS攻擊的風險
        //但實務應用時最好再加上如token等機制保護
        //參考: http://encosia.com/2008/03/27/using-jquery-to-consume-aspnet-json-web-services/
        //參考: http://weblogs.asp.net/scottgu/archive/2007/04/04/json-hijacking-and-how-asp-net-ajax-1-0-mitigates-these-attacks.aspx
        //if (Request.HttpMethod != "POST")
        //    Response.End();

        //為簡化程式碼，省略傳入參數檢查
        DataTable dt = getDataStore();
        string mode = Request["mode"];
        System.Web.Script.Serialization.JavaScriptSerializer jss = 
            new System.Web.Script.Serialization.JavaScriptSerializer();
        string id = Request["id"] ?? "";
        //傳回文章清單
        if (mode == "list")
        {
            List<PostItem> lst = new List<PostItem>();
            foreach (DataRow r in dt.Rows)
                lst.Add(new PostItem(r, false));
            Response.Write(jss.Serialize(lst));
        }
        //讀取特定文章
        else if (mode == "view")
        {
            PostItem pi = null;
            if (id == "new") //新增
                pi = new PostItem(id);
            else
                pi = new PostItem(dt.Rows.Find(id), true);
            Response.Write(jss.Serialize(pi));
        }
        //更新文章內容
        else if (mode == "update")
        {
            bool isNew = (id == "new");
            DataRow r = isNew ? dt.NewRow() : dt.Rows.Find(id);
            //若是新增，指定PK
            if (isNew)
                //找出目前最大的編號(針對模擬資料的寫法)
                r["Id"] =
                    (dt.Rows.Count == 0) ? 1 :
                    Convert.ToInt32(dt.Rows[dt.Rows.Count - 1]["Id"]) + 1;
            foreach (DataColumn c in dt.Columns)
            {
                //識別碼不能修改
                if (c.ColumnName == "Id") continue;
                r[c] = Request.Form[c.ColumnName];
            }
            //新增時，將資料加入DataTable
            if (isNew) dt.Rows.Add(r);
        }
        //刪除文章
        else if (mode == "del")
        {
            //傳入的delid會是guid1,guid2,guid3的格式
            foreach (string did in Request["did"].Split(','))
            {
                DataRow r = dt.Rows.Find(did);
                if (r != null)
                    dt.Rows.Remove(r);
            }
        }
        System.Threading.Thread.Sleep(1000);
        Response.End();
    }
}
